@@ -1,8 +1,14 @@
 #!/bin/bash
 set -ev   # display each line executed along with output
-brew update
+
+# seriously the CI needs to stop testing 99% of the time if homebrew has updates or not
+export HOMEBREW_NO_AUTO_UPDATE=1 # (please let it go, homebrew!)
+
+# brew update # (safe the CI some time)
+
 # Upgrade to python 3.x
-brew upgrade python
+# brew upgrade python # (should be fine)
+
 brew install enchant
 brew postinstall python   # this installs pip
 sudo -H pip3 install --upgrade pip setuptools wheel
